var Sensor = require("../lib/sensor.js"),
    util = require("util");


var Units = {
  "GP2Y0A21YK": {
    // https://www.sparkfun.com/products/242
    cm: {
      get: function() {
        return +((6762 / (this.voltage - 9) ) - 4).toFixed(2);
      }
    }
  },
  "GP2D120XJ00F": {
    // https://www.sparkfun.com/products/8959
    cm: {
      get: function() {
        return +((12.285 / (this.voltage-0.097) ) - 0.42).toFixed(2);
      }
    }
  },
  "GP2Y0A02YK0F": {
    // https://www.sparkfun.com/products/8958
    cm: {
      get: function() {
        return +(10650.08 * Math.pow(this.voltage, -0.935) - 10).toFixed(2);
      }
    }
  }
};


function Distance( opts ) {

  if ( !(this instanceof Distance) ) {
    return new Distance( opts );
  }

  Sensor.call( this, opts );

  var units = Units[ opts.device ];

  if (units) {
    units.inches = {
      get: function() {
        return +(this.cm * 0.39).toFixed(2);
      }
    };

    Object.defineProperties( this, units );
  }
}

util.inherits( Distance, Sensor );

module.exports = Distance;
